@@ -13,7 +13,7 @@ use constant DEBUG => $ENV{MOJO_SELENIUM_DEBUG} || 0;
 $ENV{TEST_SELENIUM} //= '0';
 $ENV{MOJO_SELENIUM_BASE_URL} ||= $ENV{TEST_SELENIUM} =~ /^http/ ? $ENV{TEST_SELENIUM} : '';
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 my $SCRIPT_NAME = File::Basename::basename($0);
 my $SCREENSHOT  = 1;
@@ -83,8 +83,15 @@ sub capture_screenshot {
 sub click_ok {
   my ($self, $selector) = @_;
   my $el = $selector ? $self->_proxy(find_element => $selector) : $self->driver->get_active_element;
-  $el->click if $el;
-  return $self->_test('ok', $el, _desc("click on $selector"));
+  my $err = 'no such element';
+
+  if ($el) {
+    eval { $self->driver->mouse_move_to_location(element => $el) } unless $el->is_displayed;
+    $err = $@ || 'unable to click';
+    $err = '' if $el->click;
+  }
+
+  return $self->_test('ok', !$err, _desc("click on $selector $err"));
 }
 
 sub current_url_is {
@@ -264,10 +271,31 @@ sub submit_ok {
   return $self->_test('ok', $el, _desc($desc, "click on $selector"));
 }
 
+sub toggle_checked_ok {
+  my ($self, $selector) = @_;
+  my $el = $self->_proxy(find_element => $selector);
+
+  if ($el) {
+    if ($el->is_displayed) {
+      $el->click;
+    }
+    else {
+      my $sel = $selector;
+      $sel =~ s!"!\\"!g;
+      $self->driver->execute_script(
+        qq[var el=document.querySelector("$sel");el.setAttribute("checked", !el.getAttribute("checked"))]
+      );
+    }
+  }
+
+  return $self->_test('ok', $el, _desc("click on $selector"));
+}
+
 sub wait_until {
   my ($self, $cb, $args) = @_;
   my $ioloop = $self->ua->ioloop;
-  my @tid;
+  my $t0     = time;
+  my ($ok, @tid);
 
   $args->{timeout}  ||= $ENV{MOJO_SELENIUM_WAIT_TIMEOUT}  || 60;
   $args->{interval} ||= $ENV{MOJO_SELENIUM_WAIT_INTERVAL} || 0.5;
@@ -279,17 +307,19 @@ sub wait_until {
       push @tid, $ioloop->recurring(
         $args->{interval},
         sub {
-          $next->() if eval { local $_ = $self->driver; $self->$cb($args) };
+          $next->(1, 1) if eval { local $_ = $self->driver; $self->$cb($args) };
           Test::More::diag("[Selenium] wait_until: $@") if $@ and ($args->{debug} or DEBUG);
         }
       );
     },
     sub {
+      $ok = $_[1];
       $ioloop->remove($_) for @tid;
     },
   )->wait;
 
-  return $self;
+  return $self if $args->{skip};
+  return $self->_test('ok', $ok, _desc($args->{desc} || "waited for @{[time - $t0]}s"));
 }
 
 sub window_size_is {
@@ -522,6 +552,15 @@ override the driver class.
 
 Where screenshots are saved.
 
+=head2 toggle_checked_ok
+
+  $self = $self->toggle_checked_ok("input[name=human]");
+
+Used to toggle the "checked" attribute either with a click event or fallback to
+javascript.
+
+TODO: The implementation might change in the future.
+
 =head1 METHODS
 
 =head2 active_element_is
@@ -739,11 +778,21 @@ See L<Selenium::Remote::WebElement/submit>.
 
 =head2 wait_until
 
-  $self = $self->wait_until(sub { my $self = shift; return 1 });
-  $self = $self->wait_until(sub { $_->get_current_url =~ /foo/ });
+  $self = $self->wait_until(sub { my $self = shift; return 1 }, \%args);
+  $self = $self->wait_until(sub { $_->get_current_url =~ /foo/ }, \%args);
+
+  # Use it as a sleep(0.8)
+  $self = $self->wait_until(sub { 0 }, {timeout => 0.8, skip => 1});
 
 Start L<Mojo::IOLoop> and run it until the callback returns true. Note that
-C<$_[0]> is C<$self> and C<$_> is L</driver>.
+C<$_[0]> is C<$self> and C<$_> is L</driver>. C<%args> is optional, but can
+contain these values:
+
+  {
+    interval => $seconds, # Default: 0.5
+    timeout  => $seconds, # Default: 60
+    skip     => $bool,    # Default: 0
+  }
 
 =head2 window_size_is
 
